#include "GetTemPreHumThread.h"
#include "BME280.h"
#include "define.h"						// ★
#include "SendRecvThread.h"				// ★

#define		GET_DATA_INTERVAL			5000
#define		BME280_ADDRESS				0x76

extern		BOOL	 SendMessagePacket(char* pcData, int iSize, LPCSTR szUserName);

//----------------------------------------------
// function
//    コンストラクタ
// parameter
//    int	iDevId		[in]スレーブアドレス/-1
//    LPSTR	szBusName   [in]Bus名(i2cdetect -lで表示される名前)/NULL
// return
//    なし
//----------------------------------------------
CGetTemPreHumThread::CGetTemPreHumThread(int iDevId, LPSTR pszBusName)
{
	if (iDevId == -1)
		iDevId = BME280_ADDRESS;
	memset(m_szUserName, 0, sizeof(m_szUserName));
	m_pCSendRecvThread = NULL;
	m_pCBME280 = new CBME280();
	m_pCBME280->Setup(iDevId, pszBusName);
}

//----------------------------------------------
// function
//    コンストラクタ
// parameter
//    LPSTR				pszUserName			[in]ユーザ名
//    CSendRecvThread	*pCSendRecvThread	[in]
//    int				iDevId				[in]スレーブアドレス/-1
//    LPSTR				szBusName			[in]Bus名(i2cdetect -lで表示される名前)/NULL
// return
//    なし
//----------------------------------------------
CGetTemPreHumThread::CGetTemPreHumThread(LPSTR pszUserName, CSendRecvThread *pCSendRecvThread, int iDevId, LPSTR pszBusName)
{
	if (iDevId == -1)
		iDevId = BME280_ADDRESS;
	strcpy(m_szUserName, pszUserName);
	m_pCSendRecvThread = pCSendRecvThread;
	m_pCBME280 = new CBME280();
	m_pCBME280->Setup(iDevId, pszBusName);
}

//----------------------------------------------
// function
//    デストラクタ
// parameter
//    なし
// return
//    なし
//----------------------------------------------
CGetTemPreHumThread::~CGetTemPreHumThread()
{
	SAFE_DELETE(m_pCBME280)
}

//----------------------------------------------
// function
//    5秒ごとに温度・気圧・湿度を取得して表示する
// parameter
//    なし
// return
//    なし
//----------------------------------------------
UINT CGetTemPreHumThread::DoWork()
{
	DWORD		dwNow;				// 取得した時刻を覚えるため
	DWORD		dwPrevTime = 0;		// 前回取得した時刻
	char		szMsg[81];
	double		dbTem, dbPre, dbHum;

	while (!m_fStopFlag)
	{
		dwNow = timeGetTime();
		if (GetdwInterval(dwNow, dwPrevTime) >= GET_DATA_INTERVAL)
		{
			dbTem = m_pCBME280->readTemp();
			dbPre = m_pCBME280->readPres();
			dbHum = m_pCBME280->readHum();
			fprintf(stderr, "\n");
			fprintf(stderr, "温度 : %7.2f ℃\n", dbTem);
			fprintf(stderr, "気圧 : %7.2f hPa\n", dbPre);
			fprintf(stderr, "湿度 : %7.2f ％\n", dbHum);
			dwPrevTime = dwNow;
			sprintf(szMsg, "温度 : %7.2f ℃ 気圧 : %7.2f hPa 湿度 : %7.2f ％", dbTem, dbPre, dbHum);
//			SendMessagePacket(szMsg, strlen(szMsg), m_szUserName);
			SendTemPreHum1Packet(dbTem, dbPre, dbHum);
//			SendTemPreHum2Packet(dbTem, dbPre, dbHum);
			SendTemPreHum3Packet(dbTem, dbPre, dbHum);
		}
		Sleep(100);
	}
	return(0);
}

//----------------------------------------------
// function
//		★温度・気圧・湿度の送信案1パケットの送信
// parameter
//		double	dbTem		[in]温度
//		double	dbPre		[in]気圧
//		double	dxHum		[in]湿度
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CGetTemPreHumThread::SendTemPreHum1Packet(double dbTem, double dbPre, double dbHum)
{
	TemPreHum1Rec *pTemPreHum1 = NULL;
	WORD		wDataLen = (WORD)sizeof(UserInfoRec) + sizeof(TemPreHumData1Rec);			//	ヘッダを除いたデータ長
	int			iPacketSize = sizeof(HeaderRec) + wDataLen;									// パケットサイズ
	BOOL		fRet = FALSE;
	int			iTem, iPre, iHum;		// 32bits符号付整数
	int			ii;

	if (m_pCSendRecvThread == NULL)
		goto L_END;
	pTemPreHum1 = (TemPreHum1Rec*)calloc(iPacketSize, sizeof(BYTE));	// パケット全体のエリアを確保

	memcpy(pTemPreHum1->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
	pTemPreHum1->header.wCommand = htons(CMD_TEMPREHUM_DATA1);			// htons ホストオーダーからネットワークオーダーへの変換(2Byte)
	pTemPreHum1->header.wDataLen = htons(wDataLen);

	DISABLE_C4996
	strcpy(pTemPreHum1->UserInfo.szUserName, m_szUserName);			// ユーザ名
	ENABLE_C4996
	iTem = (int)(RoundOff(dbTem, 2) * 100);
	for (ii = 0; ii < 4; ++ii)
		pTemPreHum1->TemPreHum.bTem[3 - ii] = (iTem >> (8 * ii)) & 0xff;
	iPre = (int)(RoundOff(dbPre, 2) * 100);
	for (ii = 0; ii < 4; ++ii)
		pTemPreHum1->TemPreHum.bPre[3 - ii] = (iPre >> (8 * ii)) & 0xff;
	iHum = (int)(RoundOff(dbHum, 2) * 100);
	for (ii = 0; ii < 4; ++ii)
		pTemPreHum1->TemPreHum.bHum[3 - ii] = (iHum >> (8 * ii)) & 0xff;

	fRet = m_pCSendRecvThread->SetSendData((char*)pTemPreHum1, iPacketSize);
L_END:
	SAFE_FREE(pTemPreHum1)
	return(fRet);
}

//----------------------------------------------
// function
//		★温度・気圧・湿度の送信案2パケットの送信
// parameter
//		double	dbTem		[in]温度
//		double	dbPre		[in]気圧
//		double	dxHum		[in]湿度
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CGetTemPreHumThread::SendTemPreHum2Packet(double dbTem, double dbPre, double dbHum)
{
	TemPreHum2Rec* pTemPreHum2 = NULL;
	WORD		wDataLen = (WORD)sizeof(UserInfoRec) + sizeof(TemPreHumData2Rec);			//	ヘッダを除いたデータ長
	int			iPacketSize = sizeof(HeaderRec) + wDataLen;									// パケットサイズ
	BOOL		fRet = FALSE;

	if (m_pCSendRecvThread == NULL)
		goto L_END;
	pTemPreHum2 = (TemPreHum2Rec*)calloc(iPacketSize, sizeof(BYTE));	// パケット全体のエリアを確保

	memcpy(pTemPreHum2->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
	pTemPreHum2->header.wCommand = htons(CMD_TEMPREHUM_DATA2);			// htons ホストオーダーからネットワークオーダーへの変換(2Byte)
	pTemPreHum2->header.wDataLen = htons(wDataLen);

	DISABLE_C4996
	strcpy(pTemPreHum2->UserInfo.szUserName, m_szUserName);			// ユーザ名
	sprintf(pTemPreHum2->TemPreHum2.szTem, "%7.2f", dbTem);
	sprintf(pTemPreHum2->TemPreHum2.szPre, "%7.2f", dbPre);
	sprintf(pTemPreHum2->TemPreHum2.szHum, "%7.2f", dbHum);
	ENABLE_C4996

	fRet = m_pCSendRecvThread->SetSendData((char*)pTemPreHum2, iPacketSize);
L_END:
	SAFE_FREE(pTemPreHum2)
	return(fRet);
}

//----------------------------------------------
// function
//		★温度・気圧・湿度の送信案3パケットの送信
// parameter
//		double	dbTem		[in]温度
//		double	dbPre		[in]気圧
//		double	dxHum		[in]湿度
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CGetTemPreHumThread::SendTemPreHum3Packet(double dbTem, double dbPre, double dbHum)
{
	char		szJson[81];
	TemPreHum3Rec *pTemPreHum3 = NULL;
	WORD		wDataLen;			//	ヘッダを除いたデータ長
	int			iSize, iPacketSize;	// パケットサイズ
	BOOL		fRet = FALSE;

	sprintf(szJson, "{\"温度\" : %7.2f, \"気圧\" : %7.2f, \"湿度\" : %7.2f}", dbTem, dbPre, dbHum);
	iSize = strlen(szJson);
	wDataLen = (WORD)sizeof(UserInfoRec) + iSize;
	iPacketSize = sizeof(HeaderRec) + wDataLen;							// パケットサイズ

	pTemPreHum3 = (TemPreHum3Rec*)calloc(iPacketSize, sizeof(BYTE));	// パケット全体のエリアを確保

	memcpy(pTemPreHum3->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
	pTemPreHum3->header.wCommand = htons(CMD_TEMPREHUM_DATA3);			// htons ホストオーダーからネットワークオーダーへの変換(2Byte)
	pTemPreHum3->header.wDataLen = htons(wDataLen);

	DISABLE_C4996
	strcpy(pTemPreHum3->UserInfo.szUserName, m_szUserName);				// ユーザ名
	ENABLE_C4996
	memcpy(pTemPreHum3->bJson, szJson, iSize);							// メッセージ
	fRet = m_pCSendRecvThread->SetSendData((char*)pTemPreHum3, iPacketSize);

	SAFE_FREE(pTemPreHum3)
	return(fRet);
}
