#include	"stdThread.h"
#include	"QueingArray.h"
#include	"MySyncObject.h"


//////////////////////////////////////////////////////////////////////
// 構築/消滅
//////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Function
//		CQueingArrayクラスの構築
//		初期化処理
// Parameter
//		long 		lQueSize	[in]キューサイズ
//		BOOL		fUseLock	[in]同期を使用するか	
// Return
//		なし
//-----------------------------------------------------------------------------
CQueingArray::CQueingArray(long lQueSize, BOOL fUseLock)
{
	int		ii;

	m_lHead	= 0;
	m_lTail = 0;
	m_lMaxQueSize		= lQueSize;						// サイズを覚える
	m_lAllocSize		= m_lMaxQueSize + 1;			// メモリを確保するサイズ
	m_fUseLock = fUseLock;
	m_pCMySyncObject = NULL;

	if (m_fUseLock == TRUE)
	{
		m_pCMySyncObject = new CMySyncObject();
		m_pCMySyncObject->Initialize();
	}

	m_pQueingData = (QueingDataRec *)calloc(m_lAllocSize, sizeof(QueingDataRec));
	for (ii = 0; ii < m_lAllocSize; ++ii)
	{
		m_pQueingData[ii].iSize = 0;
		m_pQueingData[ii].pbData = NULL;
	}
}

//-----------------------------------------------------------------------------
// Function
//		CQueingArrayクラスの破棄
//		メモリ開放処理
// Parameter
//		なし
// Return
//		なし
//-----------------------------------------------------------------------------
CQueingArray::~CQueingArray()
{
	int		ii;

	for (ii = 0; ii < m_lAllocSize; ++ii)
	{
		SAFE_FREE(m_pQueingData[ii].pbData);
		m_pQueingData[ii].iSize = 0;
	}
	SAFE_FREE(m_pQueingData)
	if (m_fUseLock == TRUE)
	{
		m_pCMySyncObject->Uninitialize();
		SAFE_DELETE(m_pCMySyncObject)
	}
}


//-----------------------------------------------------------------------------
// Function
//		キューにデータを入れる
// Parameter
//		LPBYTE	pbData	[in]データ
//		int		iSize	[in]データサイズ
// Return
//		なし
//-----------------------------------------------------------------------------
void CQueingArray::PushData(LPBYTE pbData, int iSize)
{
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->Lock();

	if (m_pQueingData[m_lTail].pbData != NULL)
	{
		m_pQueingData[m_lTail].iSize = 0;
		SAFE_FREE(m_pQueingData[m_lTail].pbData)
	}
	m_pQueingData[m_lTail].pbData = (LPBYTE)calloc(iSize, sizeof(BYTE));
	memcpy(m_pQueingData[m_lTail].pbData, pbData, iSize);
	m_pQueingData[m_lTail].iSize = iSize;

	m_lTail++;
	m_lTail %= m_lAllocSize;
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->UnLock();
}


//-----------------------------------------------------------------------------
// Function
//		キューからデータを取り出す
// Parameter
//		LPBYTE	*ppbData	[out]データ
//		int		*piSize		[out]データサイズ
// Return
//		なし
//-----------------------------------------------------------------------------
void	CQueingArray::PopData(LPBYTE *ppbData, int *piSize)
{
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->Lock();

	*ppbData = NULL;
	*piSize = 0;
	if(m_pQueingData[m_lHead].iSize > 0)
	{
		*piSize = m_pQueingData[m_lHead].iSize;
		*ppbData = (LPBYTE)calloc(*piSize, sizeof(BYTE));
		memcpy(*ppbData, m_pQueingData[m_lHead].pbData, *piSize);
	}
	m_pQueingData[m_lHead].iSize = 0;
	SAFE_FREE(m_pQueingData[m_lHead].pbData)

	m_lHead++;
	m_lHead %= m_lAllocSize;

	if (m_fUseLock == TRUE)
		m_pCMySyncObject->UnLock();
}


//-----------------------------------------------------------------------------
// Function
//		キューイングされている個数を取得する
// Parameter
//		なし
// Return
//		キューイングされている個数
//-----------------------------------------------------------------------------
long	 CQueingArray::GetSize()
{
	long	n;

	if (m_fUseLock == TRUE)
		m_pCMySyncObject->Lock();
	n = (m_lTail >= m_lHead) ? (m_lTail - m_lHead) : ((m_lAllocSize - m_lHead) + m_lTail);
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->UnLock();
	return(n);
}


//-----------------------------------------------------------------------------
// Function
//		キューイングすることが可能か
//		キューの空きが１つ以上あるか
// Parameter
//		なし
// Return
//		TRUE/FALSE
//-----------------------------------------------------------------------------
BOOL CQueingArray::CheckEnablePush()
{
	long	n;

	if (m_fUseLock == TRUE)
		m_pCMySyncObject->Lock();
	n = (m_lTail < m_lHead) ? (m_lHead - m_lTail) : ((m_lAllocSize - m_lTail) + m_lHead);
	if(n > 0)
		--n;
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->UnLock();

	if(n > 0)
		return(TRUE);
	else
		return(FALSE);
}


//-----------------------------------------------------------------------------
// Function
//		キューを空にする
// Parameter
//		なし
// Return
//		なし
//-----------------------------------------------------------------------------
void CQueingArray::Dump()
{
	int		ii;

	if (m_fUseLock == TRUE)
		m_pCMySyncObject->Lock();

	m_lHead	= 0;
	m_lTail = 0;
	for(ii = 0 ; ii < m_lAllocSize ; ++ii)
	{
		m_pQueingData[ii].iSize = 0;
		SAFE_FREE(m_pQueingData[ii].pbData)
	}
	if (m_fUseLock == TRUE)
		m_pCMySyncObject->UnLock();
}
