#include "SendRecvThread.h"
#include "RingBuff.h"
#include "define.h"				//
#include "QueingArray.h"		//
#include "GetTemPreHumThread.h"
#include "MySyncObject.h"

#define	RECV_BUFF_SIZE		(1024 * 64)		// ★受信リングバッファのサイズ
#define SEND_BUFF_QUE_SIZE	1000			// ★送信バッファとして確保するキューの数
// 送信速度
#define		SEND_BPS	(1000000.0)			// 送信速度1Mbps

//----------------------------------------------
// function
//    コンストラクタ
// parameter
//    ConnectionInfoRec *pConInfo	[in]機能に必要な情報
// return
//    なし
//----------------------------------------------
CSendRecvThread::CSendRecvThread(ConnectionInfoRec* pConInfo)
{
	m_pConInfo = pConInfo;
	m_fIamZombie = FALSE;

	m_pCRingBuffRecv = new CRingBuff(RECV_BUFF_SIZE);			// 受信データ格納用エリア
	m_pCQueingArraySend = new CQueingArray(SEND_BUFF_QUE_SIZE);	// ★送信バッファ
	m_pCMySyncObjectSendWrite = new CMySyncObject();			// 送信バッファへの書き込み同期用
	m_pCMySyncObjectSendWrite->Initialize();
	m_pCGetTemPreHumThread = new CGetTemPreHumThread(pConInfo->szUserName, this);
	m_pCGetTemPreHumThread->Begin();

	m_dwPrevSentTime = 0;									//
	m_iSendInterval = 0;									//
}

//----------------------------------------------
// function
//    デストラクタ
// parameter
//    なし
// return
//    なし
//----------------------------------------------
CSendRecvThread::~CSendRecvThread()
{
	if (m_pCGetTemPreHumThread != NULL)
	{
		m_pCGetTemPreHumThread->End();
		m_pCGetTemPreHumThread->WaitForEnd();
		SAFE_DELETE(m_pCGetTemPreHumThread)
	}
	SAFE_DELETE(m_pCRingBuffRecv)		// 受信バッファの破棄
	SAFE_DELETE(m_pCQueingArraySend)	// ★送信バッファの破棄
	m_pCMySyncObjectSendWrite->Uninitialize();
	SAFE_DELETE(m_pCMySyncObjectSendWrite)
}

#define	RCVBUFSIZE		(1024)			// 一回に読む最大受信サイズ
//----------------------------------------------
// function
//    機能を記述した関数(DoSendに記述してあった内容を記述する)
// parameter
//    なし
// return
//    0:正常 -1:エラー発生
//----------------------------------------------
UINT CSendRecvThread::DoWork()
{
	BOOL		fRet = TRUE;
	pollfd		fds[1] = { 0 };					// 接続済みソケットを調べるので１つ
	char		*pcData = NULL;					// 未送信データを格納するエリア
	int			iSendSize = 0;					// 未送信データのサイズ
	char	    szRecvBuffer[RCVBUFSIZE + 1];	// 受信バッファ
	int			iRecvSize;
	DWORD		dwNow;							// ★送信チェックした時刻を覚えるため
	sockaddr_storage	PeerAddr;				// ★相手アドレス情報
	socklen_t	iPeerLen;						// ★相手アドレス情報のサイズ
	char		szPeerAddr[NI_MAXHOST];			// ★相手アドレス,ポート番号をgetnameinfoで取得
	char		szServiceNo[NI_MAXSERV];		// ★

	while (!m_fStopFlag)
	{
		// 未送信のデータがなければ送信したいデータがあるか調べる
		if (iSendSize == 0)
			iSendSize = GetSendData(&pcData);

		// pollfdにソケットと調査イベントをセット
		fds[0].fd = m_pConInfo->fdClient;
		// 受信と相手側からの切断イベントを設定
		fds[0].events = POLLIN | POLLRDHUP;	
		// ★送信可能時刻かつ未送信データがあるときだけ送信可能検査を追加設定
		// ★チェックのために現在時刻を取得する
		dwNow = timeGetTime();
		if ((iSendSize > 0) && (CanSendNow(dwNow) == TRUE))
			fds[0].events |= POLLOUT;
		poll(fds, 1, 10);

		// エラー発生処理
		if (fds[0].revents & POLLERR)
		{
			DispNetErrorMsg("Err:DoWork()");
			fRet = FALSE;
			break;
		}
		// 切断発生処理
		if (fds[0].revents & POLLRDHUP)
		{
			DispNetErrorMsg("Disconnected");
			fRet = FALSE;
			break;
		}
		// 受信処理処理(切断、エラーチェック、受信データは読み捨てる)
		if (fds[0].revents & POLLIN)
		{
			// 受信リングバッファに空きがあれば取得する
			// UDPは受信パケットサイズより小さなバッファでrecvfromを呼ぶと残りが破棄されるので
			// 空きが受信パケットサイズより大きいときにrecvfromする
			if (m_pCRingBuffRecv->GetWriteableSize() >= RCVBUFSIZE)
			{
				memset(szRecvBuffer, 0, sizeof(szRecvBuffer));	// 受信バッファの初期化
				iPeerLen = sizeof(PeerAddr);					// 相手アドレス情報を格納する構造体のサイズ
// UDPは0バイトの受信がありうる
				if ((iRecvSize = recvfrom(m_pConInfo->fdClient, szRecvBuffer, RCVBUFSIZE, 0,
					(sockaddr*)&PeerAddr, &iPeerLen)) < 0)
				{
					DispNetErrorMsg("Err:recvfrom");
					fRet = FALSE;
					break;
				}
				else
				{
					// 送信元の表示
					getnameinfo((sockaddr*)&PeerAddr, iPeerLen, szPeerAddr, NI_MAXHOST,
						szServiceNo, NI_MAXSERV, NI_NUMERICHOST | NI_NUMERICSERV);
					fprintf(stderr, "recv from Addr:%s Port:%s\n:", szPeerAddr, szServiceNo);
					// 受信したデータを一旦溜めます(リングバッファ)
					SetRecvData(szRecvBuffer, iRecvSize);
				}
			}
		}
		// 送信処理
		if (fds[0].revents & POLLOUT)
		{
			if (sendto(m_pConInfo->fdClient, pcData, iSendSize, 0,
				m_pConInfo->ai_addr, (int)m_pConInfo->ai_addrlen) != iSendSize)
			{
				DispNetErrorMsg("Err:sendto");
				fRet = FALSE;
				break;
			}
			fprintf(stderr, "sendto()送信サイズ %d", iSendSize);
			SAFE_FREE(pcData)							// 未送信データなしにセット
		// 送信時刻と次回送信までの時間をセット
			m_dwPrevSentTime = dwNow;
			m_iSendInterval = CalcNextSendInterval(iSendSize);
			iSendSize = 0;
		}
		// 受信データ解析
		if (AnalyzeRecvData() == -1)					// エラー発生時は処理を終了する
		{
			DispNetErrorMsg("Err:Packet format");
			fRet = FALSE;
			break;
		}
	}
	m_fIamZombie = TRUE;
	SAFE_FREE(pcData)
	iSendSize = 0;
	return((fRet == TRUE) ? 0 : -1);
}

//----------------------------------------------
// function
//		受信データの解析
// parameter
//		なし
// retun
//		0:パケットが揃っていない
//		1:パケットを受信したので処理を行った
//		-1:エラーが発生した
//----------------------------------------------
int CSendRecvThread::AnalyzeRecvData()
{
	int				iRet = 0;
	HeaderRec		Header;
	WORD			wCmd, wDataLen;
	int				iPacketSize;
	LPBYTE			pbPacketData = NULL;
	int				iSize = m_pCRingBuffRecv->GetReadableSize();	// 受信バッファに溜まっているデータ数の取得

	if (iSize < sizeof(HeaderRec))			// ヘッダサイズに満たないときは何もしない
		goto L_END;
	// ヘッダ部を借り読み込みする
	m_pCRingBuffRecv->ReadWithoutUpdateHeadPoint((LPBYTE)&Header, sizeof(HeaderRec));
	// ヘッダ部の解析
	if (memcmp(Header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING)) != 0)
	{
		iRet = -1;							// 識別子が違うのでエラー
		goto L_END;
	}
	wCmd = ntohs(Header.wCommand);			// ntohs ネットオーダーをホストオーダーに変換
	wDataLen = ntohs(Header.wDataLen);
	iPacketSize = sizeof(Header) + wDataLen;
	if (iSize < iPacketSize)				// パケットサイズに満たないときは何もしない
		goto L_END;
	// パケット分のデータを取り出す
	pbPacketData = (LPBYTE)calloc(iPacketSize, sizeof(BYTE));
	m_pCRingBuffRecv->ReadWithoutUpdateHeadPoint(pbPacketData, iPacketSize);

	switch (wCmd)
	{
		case CMD_MSG_DATA: 						// メッセージパケットに対する処理を行う
			iRet = SetMessagePacket(pbPacketData, iPacketSize);
			break;
		default:								// 知らないコマンドなのでエラー
			iRet = -1;
			break;
	}
	SAFE_FREE(pbPacketData)
	if (iRet == 1)
		m_pCRingBuffRecv->UpdateHeadPoint(iPacketSize);
L_END:
	return(iRet);
}

//----------------------------------------------
// function
//    チャットパケットの処理
//	  メッセージの表示
// parameter
//    LPBYTE pbPacketData	[in]チャットパケット
//	  int iPacketSize		[in]パケットサイズ
// return
//    1:正常 -1:エラー発生
//----------------------------------------------
int CSendRecvThread::SetMessagePacket(LPBYTE pbPacketData, int iPacketSize)
{
	MsgDataRec* pmsgData = (MsgDataRec*)pbPacketData;
	int			iDataSize = iPacketSize - sizeof(HeaderRec);
	int			iMsgSize = iDataSize - sizeof(UserInfoRec);
	LPBYTE		pbMsg = (LPBYTE)calloc(iMsgSize + 1, sizeof(BYTE));

	// pmsgData->bMsgData に、UTF-8で格納されている(NULLターミネートなし)
	memcpy(pbMsg, pmsgData->bMsgData, iMsgSize);
	fprintf(stdout, "%s:%s\n", pmsgData->UserInfo.szUserName, (char*)pbMsg);	// 受信データの表示
	fflush(stdout);
	SAFE_FREE(pbMsg)
	return(1);
}

//----------------------------------------------
// function
//		★送信データの設定
// parameter
//		char		*pcData	[in]送信データ
//		int		iSize			[in]データ長
// return
//		TRUE/FALSE
//----------------------------------------------
BOOL CSendRecvThread::SetSendData(char* pcData, int iSize)
{
	BOOL	fRet = FALSE;

	m_pCMySyncObjectSendWrite->Lock();
	if (m_pCQueingArraySend->CheckEnablePush() == FALSE)
		goto L_END;
	m_pCQueingArraySend->PushData((LPBYTE)pcData, iSize);
	fRet = TRUE;

L_END:
	m_pCMySyncObjectSendWrite->UnLock();
	return(fRet);
}

//----------------------------------------------
// function
//		★送信データの取得
// parameter
//		char **ppcData	[in/out]送信データ
// return
//		データ長
//----------------------------------------------
int CSendRecvThread::GetSendData(char** ppcData)
{
	int		iSize = m_pCQueingArraySend->GetSize();

	if (iSize > 0)
		m_pCQueingArraySend->PopData((LPBYTE*)ppcData, &iSize);
	return(iSize);
}

//----------------------------------------------
// function
//		受信データの設定
// parameter
//		char	*pcData	[in]受信データ
//		int		iSize	[in]データ長
// return
//		TRUE/FALSE
//----------------------------------------------
BOOL CSendRecvThread::SetRecvData(char* pcData, int iSize)
{
	BOOL	fRet = FALSE;

	// リングバッファに空きがあるか調べる
	if (m_pCRingBuffRecv->GetWriteableSize() < iSize)
		goto L_END;
	m_pCRingBuffRecv->Write((LPBYTE)pcData, iSize);
	fRet = TRUE;

L_END:
	return(fRet);
}

//----------------------------------------------
// function
//		受信データの取得
// parameter
//		char **ppcData	[in/out]受信データ
// return
//		データ長
//----------------------------------------------
int CSendRecvThread::GetRecvData(char** ppcData)
{
	// リングバッファに格納されているサイズを調べる
	int		iSize = m_pCRingBuffRecv->GetReadableSize();

	if (iSize > 0)
	{
		*ppcData = (char*)calloc(iSize, sizeof(char));
		m_pCRingBuffRecv->Read((LPBYTE)*ppcData, iSize);
	}
	return(iSize);
}

//----------------------------------------------
// function
//    このスレッドはゾンビ状態か
//	  別スレッドから参照される
// parameter
//    なし
// return
//    0:正常 -1:エラー発生
//----------------------------------------------
BOOL CSendRecvThread::IsZombie()
{
	BOOL	fRet;

	fRet = m_fIamZombie;
	return(fRet);
}

//----------------------------------------------
// function
//		次の送信までの間隔(msec)を求める
// parameter
//		int iSentSize	[in]送信したバイト数
// return
//		msec
//----------------------------------------------
int CSendRecvThread::CalcNextSendInterval(int iSentSize)
{
	int		iInterval = 0;
	double	dbBytePerSec = SEND_BPS / 8.0;
	double	dbCountPerSec;

	if (iSentSize == 0)
		goto L_END;
	dbCountPerSec = dbBytePerSec / (double)iSentSize;
	iInterval = int(1000.0 / dbCountPerSec);

L_END:
	return(iInterval);
}


//----------------------------------------------
// function
//		今送信可能時刻か
// parameter
//		DWORD dwNow		[in]現在時刻
// return
//		TRUE/FALSE
//----------------------------------------------
BOOL CSendRecvThread::CanSendNow(DWORD dwNow)
{
	BOOL	fRet = FALSE;

	if (GetdwInterval(dwNow, m_dwPrevSentTime) >= (DWORD)m_iSendInterval)
		fRet = TRUE;

	return(fRet);
}
