#pragma once
#include "ThreadJob.h"

typedef struct
{
	SOCKET			fdClient;				// UDPソケット
	sockaddr		*ai_addr;				// 相手アドレス情報(sendtoのパラメータ)
	size_t			ai_addrlen;
	char			szUserName[81];			// ★UserName UTF-8(nullターミネートあり)
} ConnectionInfoRec;

class CRingBuff;
class CQueingArray;							// ★
class CGetTemPreHumThread;
class CMySyncObject;


class CSendRecvThread :
    public CThreadJob
{
public:
	CSendRecvThread(ConnectionInfoRec* pConInfo);	// パラメータをコンストラクタで渡す
	~CSendRecvThread();

	// 基底クラスの関数をオーバーライドする
	// C++11で明示的にoverrideを書くことが出来るようになりました
	// 基底クラスの当該関数にvirtualが書いていないとエラーを出してくれます
	UINT DoWork() override;							// DoSendで実施している内容を記述
	BOOL SetSendData(char* pcData, int iSize);		// 送信データの設定			
	int GetRecvData(char** ppcData);				// 受信データの取得
	BOOL IsZombie();								// このスレッドはゾンビ状態か

private:
	ConnectionInfoRec* m_pConInfo;					// コンストラクタで渡されるパラメータを格納
													// このスレッド実行中領域が確保されていること
	BOOL m_fIamZombie;								// ゾンビ状態かどうかを保持

	CQueingArray* m_pCQueingArraySend;				// ★送信データ格納用バッファ
	CRingBuff* m_pCRingBuffRecv;					// 受信データ格納用エリア

	DWORD	m_dwPrevSentTime;						// 前回送信した時刻(msec)
	int		m_iSendInterval;						// 次回送信までの間隔(msec)

	CGetTemPreHumThread* m_pCGetTemPreHumThread;

	CMySyncObject* m_pCMySyncObjectSendWrite;		// 送信バッファへの書き込み同期用


	int GetSendData(char** ppcData);				// 送信データの取得
	BOOL SetRecvData(char* pcData, int iSize);		// 受信データの設定			

	int AnalyzeRecvData();							// 受信リングバッファに溜まったデータの解析
	int SetMessagePacket(LPBYTE pbPacketData, int iPacketSize);		// チャットパケットの処理

	int CalcNextSendInterval(int iSentSize);		// 次回送信までの間隔
	BOOL CanSendNow(DWORD dwNow);					// 送信して良い時刻になったか
};

