#include	"stdThread.h"
#include	"SendRecvThread.h"	// 送受信スレッドを使うため
#include	"define.h"			// パケットの宣言
#include    <ifaddrs.h>			// ★
#include    <net/if.h>			// ★

// メニューで使用する文字
#define		CMD_QUIT_CHAR				'q'
#define		CMD_SEND_MSG_CHAR			'm'

// 送信元インターフェースの設定
#define			IF_LOOP_BACK		1
#define			IF_TTL				32

// メッセージ送信間隔
#define		SEND_INTERVAL		(5 * 1000)

// 関数の宣言
BOOL CreateSocket(LPCSTR szDestAddress, WORD wDestPort, LPCSTR pszIFName, WORD wSrcPort, LPCSTR pszUserName);	// ★UDPソケットの作成
BOOL BindSocket(SOCKET fd, WORD wClientPort, int iFamily, int iSocktype, int iProtocol);	// bindの実施
void SetSendSocketOption(SOCKET fd, int iFamily, LPCSTR szIFName);							// ★送信元インターフェースの設定
BOOL DestroySocket(SOCKET& fd);								// ソケットの破棄
void Stop();												// すべてのソケットを破棄する
void DispMenu();											// メニューの表示
int	 GetKeyString(LPSTR pszString, int iSize);				// キーボード入力文字列の取得
BOOL KillZombei();											// ゾンビ状態のスレッドを破棄する
BOOL CheckAndSendMessagePacket(char* pcData, int iSize, LPCSTR szUserName);	// ★
BOOL SendMessagePacket(char* pcData, int iSize, LPCSTR szUserName);	// メッセージをパケット化して送信
sockaddr* GetInterfaceInfo(int iFamily, LPCSTR szIFName, DWORD* pdwIndex);	// ★指定されたインターフェースの情報を取得する

// 変数の宣言
CSendRecvThread* m_pCSendRecvThread = NULL;
ConnectionInfoRec* m_pConInfo = NULL;

int main(int argc, char* argv[])
{
	char	szKeyInBuff[81];
	int		iSize = 0;							// ★メッセージの長さ
	LPBYTE	pbDest = NULL;						// ★メッセージを格納するエリア(NULLターミネートなし)
	LPCSTR	pszUserName;
	WORD	wClientPort = 0;					// 自ポート
	LPSTR	pszIFName = NULL;					// ★送信元インターフェース名
	DWORD	dwPevTime = 0, dwNow;				// ★前回メッセージを送信した時刻
	DWORD	dwInterval = SEND_INTERVAL;			// ★送信間隔

// ★起動パラメータチェック
	if ((argc < 4) || (argc > 6))
	{
		fprintf(stderr, "Usage:  %s <Multicast IP> <DestPort> <UserName> [<InterfaceName>] [<SrcPort>]\n", argv[0]);
		goto L_END;
	}
	// ユーザ名をpszUserNameに格納
	pszUserName = argv[3];
	if ((argc >= 5))					// ★IF名
		pszIFName = argv[4];
	if (argc == 6)						// ★送信元ポート
		wClientPort = (WORD)atol(argv[5]);

	// ★ソケットの作成 送受信スレッドを開始する
	if (CreateSocket(argv[1], (WORD)atol(argv[2]), pszIFName, wClientPort, pszUserName) == FALSE)
		goto L_END;
	// メニューの表示
	DispMenu();
	while (1)
	{
		if (KillZombei() == TRUE)			// 切断してたら終了
			break;
		Sleep(10);							// CPUの負荷軽減の為
		// キーボードから入力された文字列が'q'なら終了
		// ‘m’ならメッセージを入力後は送信バッファに格納する		
		switch (GetKeyString(szKeyInBuff, sizeof(szKeyInBuff)))
		{
			case 0:							// 入力なし、何もしない
				break;
			case CMD_QUIT_CHAR:				// 'q'入力終了
				goto L_END;
			case CMD_SEND_MSG_CHAR:			// 'm'入力szKeyInBuffに文字列が格納されている
				if(strlen(szKeyInBuff) > 0)
				{
					iSize =  (int)strlen(szKeyInBuff);
					pbDest = (LPBYTE)calloc(iSize, sizeof(BYTE));
					memcpy(pbDest, szKeyInBuff, iSize);
					SendMessagePacket((char*)pbDest, iSize, pszUserName);
					dwPevTime = timeGetTime();	// ★送信した時刻を覚える
				}
				else
				{
					// 空文字列の時は送信終了
					SAFE_FREE(pbDest)
					iSize = 0;
				}
				// メニューの表示
				DispMenu();
				break;
		}
		// ★5秒経過してたら送信
		dwNow = timeGetTime();
		if (GetdwInterval(dwNow, dwPevTime) >= dwInterval)
		{
			CheckAndSendMessagePacket((char*)pbDest, iSize, pszUserName);
			dwPevTime = dwNow;
		}
		
	}
L_END:
	SAFE_FREE(pbDest)
	// 送受信スレッドの破棄、切断とすべてのソケットの破棄
	Stop();
	return(0);
}


//----------------------------------------------
// function
//		UDPソケットの作成
//		★送信ソケットの設定
//		送受信スレッドを開始する
// parameter
//		LPCSTR      szDestAddress	[in]送信先アドレス(マルチキャストアドレス)
//		WORD        wDestPort		[in]送信先ポート
//		LPCSTR		pszIFName		[in]送信元IF名/NULL
//		WORD		wSrcPort		[in]送信元ポート/0
// return
//		TRUE/FALSE
//----------------------------------------------
BOOL CreateSocket(LPCSTR szDestAddress, WORD wDestPort, LPCSTR pszIFName, WORD wClientPort, LPCSTR pszUserName)
{
	BOOL		fRet = TRUE;
	SOCKET		fd = INVALID_SOCKET;
	addrinfo	hints, * pres = NULL, * pTemp = NULL;
	char		szPort[NI_MAXSERV];

	memset(&hints, 0, sizeof(hints));
	hints.ai_socktype = SOCK_DGRAM;			// UDP
	DISABLE_C4996
	sprintf(szPort, "%d", wDestPort);		// ポート番号=サービス
	ENABLE_C4996
	// アドレス, ポートを指定してgetaddrinfoを実行 接続先アドレス情報を得る
	if (getaddrinfo(szDestAddress, szPort, &hints, &pres) == 0)
	{
		pTemp = pres;
		while (pTemp != NULL)
		{
			// socketを実行
			if ((fd = socket(pTemp->ai_family, pTemp->ai_socktype, pTemp->ai_protocol)) == INVALID_SOCKET)
				goto L_NEXT;
			// 送信ソケットの設定
			SetSendSocketOption(fd, pTemp->ai_family, pszIFName);
			// socketが作れたのでこのソケット採用することにして抜ける
			fRet = TRUE;
			break;
L_NEXT:
			pTemp = pTemp->ai_next;
		}
	}
	if (fd == INVALID_SOCKET)		// ソケット作成に失敗
	{
		DispNetErrorMsg("Err:socket");
		fRet = FALSE;
		goto L_END;
	}
// 自ポートが指定されたときはbindを実施する(受信にも使用する場合指定することが多い)
// ポート番号をbindしないときは自ポートは動的になります(0でbindしても同様です)
	if (wClientPort != 0)
		if ((fRet = BindSocket(fd, wClientPort, pTemp->ai_family, pTemp->ai_socktype, pTemp->ai_protocol)) == FALSE)
			goto L_END;

	// 送受信スレッドを開始する
	m_pConInfo = (ConnectionInfoRec*)calloc(1, sizeof(ConnectionInfoRec));
	m_pConInfo->fdClient = fd;								// UDPソケット
	m_pConInfo->ai_addr = (sockaddr*)calloc(pTemp->ai_addrlen, sizeof(BYTE));	// 送信先アドレス
	m_pConInfo->ai_addrlen = pTemp->ai_addrlen;
	memcpy(m_pConInfo->ai_addr, pTemp->ai_addr, pTemp->ai_addrlen);
	strcpy(m_pConInfo->szUserName, pszUserName);
	m_pCSendRecvThread = new CSendRecvThread(m_pConInfo);
	m_pCSendRecvThread->Begin();
L_END:
	if (pres != NULL)
		freeaddrinfo(pres);
	if ((fRet == FALSE) && (fd != INVALID_SOCKET))
	{
		DestroySocket(fd);
	}
	return(fRet);
}

//----------------------------------------------
// function
//		bindの実施
//		自ポートが指定されたときはbindを実施する(受信にも使用する場合指定することが多い)
// parameter
//		SOCKET	fd			[in]ソケット
//		WORD	wClientPort	[in]自ポート
//		int		iFamily		[in]PF_xxx
//		int		iSocktype	[in]SOCK_xxx
//		int		iProtocol	[in]0 or IPPROTO_xxx for IPv4 and IPv6
// return
//		TRUE/FALSE
//----------------------------------------------
BOOL BindSocket(SOCKET fd, WORD wClientPort, int iFamily, int iSocktype, int iProtocol)
{
	BOOL		fRet = TRUE;
	addrinfo	hints, * pres = NULL, * pTemp = NULL;
	char		szPort[NI_MAXSERV];
	int			on;

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = iFamily;
	hints.ai_socktype = iSocktype;
	hints.ai_protocol = iProtocol;
	// すべてのインターフェースで受信可
	hints.ai_flags = AI_PASSIVE;
	DISABLE_C4996
	sprintf(szPort, "%d", wClientPort);			// ポート番号=サービス
	ENABLE_C4996

	if (getaddrinfo(NULL, szPort, &hints, &pres) == 0)
	{
#ifdef IPV6_V6ONLY
		// IPv6ソケットでIPv4射影アドレスを使用しないように設定
		if (iFamily == AF_INET6)
		{
			on = 1;
			if (setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (char*)&on, sizeof(on)) < 0)
			{
				DispNetErrorMsg("Err:setsockopt");
				fRet = FALSE;
				goto L_END;
			}
		}
#endif
		if (bind(fd, pres->ai_addr, (int)pres->ai_addrlen) == SOCKET_ERROR)
		{
			DispNetErrorMsg("Err:bind");
			fRet = FALSE;
		}
	}
	if (pres != NULL)
		freeaddrinfo(pres);

L_END:
	return(fRet);
}

//----------------------------------------------
// function
//		★送信ソケットの設定
//		送信元インターフェース, ループバック, TTL/HopLimitsの設定
// parameter
//		SOCKET		fd			[in]ソケット
//		int			iFamily		[in]アドレスファミリ
//		LPCSTR		szIFName	[in]送信元インターフェース名/NULL
// return
//		TRUE/FALSE
//----------------------------------------------
void SetSendSocketOption(SOCKET fd, int iFamily, LPCSTR szIFName)
{
	sockaddr* pAddr = NULL;
	int			iRet;
	DWORD		dwFIndex;

	// 1.送信元インターフェースの設定(インターフェース名が指定されたとき)
	if ((pAddr = GetInterfaceInfo(iFamily, szIFName, &dwFIndex)) != NULL)
	{
		if (iFamily == AF_INET)
			iRet = setsockopt(fd, IPPROTO_IP, IP_MULTICAST_IF, (char*)&(((sockaddr_in*)pAddr)->sin_addr), sizeof(in_addr));
		else if (iFamily == AF_INET6)
			iRet = setsockopt(fd, IPPROTO_IPV6, IPV6_MULTICAST_IF, (char*)&dwFIndex, sizeof(dwFIndex));
	}
	// 2.ループバックの設定
	DWORD		dwLoop = IF_LOOP_BACK;
	if (iFamily == AF_INET)
		iRet = setsockopt(fd, IPPROTO_IP, IP_MULTICAST_LOOP, (char*)&dwLoop, sizeof(dwLoop));
	else if (iFamily == AF_INET6)
		iRet = setsockopt(fd, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, (char*)&dwLoop, sizeof(dwLoop));

	// 3.TTL/HopLimitsの設定
	DWORD		dwTTL = IF_TTL;
	if (iFamily == AF_INET)
		iRet = setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, (char*)&dwTTL, sizeof(dwTTL));
	else if (iFamily == AF_INET6)
		iRet = setsockopt(fd, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, (char*)&dwTTL, sizeof(dwTTL));

	SAFE_FREE(pAddr)
}


//----------------------------------------------
// function
//    ソケットの破棄
// parameter
//    SOCKET		&fd[in/out]破棄するソケット
// return
//    TRUE/FALSE
//----------------------------------------------
BOOL DestroySocket(SOCKET& fd)
{
	fprintf(stderr, "DestroySocket()\n");
	if (fd != INVALID_SOCKET)
	{
		shutdown(fd, SHUT_RDWR);     // 受信も送信も停止
		close(fd);
		fd = INVALID_SOCKET;
	}
	return(TRUE);
}

//----------------------------------------------
// function
//    すべてのソケットの切断、破棄
// parameter
//    なし
// return
//    なし
//----------------------------------------------
void Stop()
{
	fprintf(stderr, "Stop()\n");
	if (m_pCSendRecvThread != NULL)
	{
		m_pCSendRecvThread->End();
		m_pCSendRecvThread->WaitForEnd();
		SAFE_DELETE(m_pCSendRecvThread)

		DestroySocket(m_pConInfo->fdClient);
		SAFE_FREE(m_pConInfo->ai_addr)			// ★
		SAFE_FREE(m_pConInfo)
	}
}

//----------------------------------------------
// function
//    メニューの表示
// parameter
//    なし
// return
//    なし
//----------------------------------------------
void DispMenu()
{
	fprintf(stdout, "%c:quit %c:メッセージ送信 : ", CMD_QUIT_CHAR, CMD_SEND_MSG_CHAR);
	fflush(stdout);
}

#if 0
//----------------------------------------------
// function
//    キーボード入力(改行まで)の取得
//		入力がないときはすぐにリターン
// parameter
//    LPSTR		pszString	[in/out]データ格納用バッファ
//    int		iSize		[in]バッファのサイズ
// return
//    0:入力なし CMD_QUIT_CHAR:終了 CMD_SEND_MSG_CHAR:メッセージ送信
//----------------------------------------------
int GetKeyString(LPSTR pszString, int iSize)
{
	BOOL			fRet = FALSE;
	int				iRet = 0;
	pollfd			fds[1] = { 0 };
	fds[0].fd = 0;							// 0:標準入力 1:標準出力 2:標準エラー
	fds[0].events = POLLIN;					// Enterキー入力を対象に
	poll(fds, 1, 10);
	if (fds[0].revents & POLLIN)			// ENTERキー入力があった
	{
		fgets(pszString, iSize, stdin);		// ENTERキーまで読み込む
		switch (pszString[0])
		{
			case CMD_QUIT_CHAR:
				iRet = CMD_QUIT_CHAR;
				break;
			case CMD_SEND_MSG_CHAR:
				fprintf(stderr, "\n送信メッセージを入力してください : ");
				fgets(pszString, iSize, stdin); 	// ENTERキーまで読み込む
				if (strlen(pszString) > 0)
				{
					// 改行まで読み込んでいるので
					if (pszString[strlen(pszString) - 1] == '\n')
						pszString[strlen(pszString) - 1] = 0;
					iRet = CMD_SEND_MSG_CHAR;
				}
				break;
			default:
				break;
		}
	}
	return(iRet);
}
#else
//----------------------------------------------
// function
//    キーボード入力(改行まで)の取得
//		入力がないときはすぐにリターン
//　　Windows版と同じコード、同じ動作
//      但し、端末によってはモード変更ができないので上手く動かない
//      例 VSのデバッグコンソール
// parameter
//    LPSTR		pszString	[in/out]データ格納用バッファ
//    int		iSize		[in]バッファのサイズ
// return
//    0:入力なし CMD_QUIT_CHAR:終了 CMD_SEND_MSG_CHAR:メッセージ送信
//----------------------------------------------
int GetKeyString(LPSTR pszString, int iSize)
{
	int	    iRet = 0;

	memset(pszString, 0, iSize);
	if (kbhit())
	{
		switch (getch())
		{
		case CMD_QUIT_CHAR:
			iRet = CMD_QUIT_CHAR;
			break;
		case CMD_SEND_MSG_CHAR:
			fprintf(stdout, "\n送信メッセージを入力してください : ");
			fgets(pszString, iSize, stdin);
			if (strlen(pszString) > 0)
			{
				// 改行まで読み込んでいるので
				if (pszString[strlen(pszString) - 1] == '\n')
					pszString[strlen(pszString) - 1] = 0;
				iRet = CMD_SEND_MSG_CHAR;
			}
			break;
		default:
			break;
		}
	}
	return(iRet);
}
#endif

//----------------------------------------------
// function
//	切断済みのCSendRecvThreadを破棄する
// parameter
//	なし
// return
//	TRUE:破棄した/FALSE:破棄すべきものがなかった
//----------------------------------------------
BOOL KillZombei()
{
	BOOL	fRet = FALSE;

	if (m_pCSendRecvThread != NULL)
	{
		if (m_pCSendRecvThread->IsZombie() == TRUE)
		{
			m_pCSendRecvThread->End();
			m_pCSendRecvThread->WaitForEnd();
			SAFE_DELETE(m_pCSendRecvThread);

			DestroySocket(m_pConInfo->fdClient);
			SAFE_FREE(m_pConInfo->ai_addr)			// ★
			SAFE_FREE(m_pConInfo)
			fRet = TRUE;
		}
	}
	return(fRet);
}

//----------------------------------------------
// function
//		★メッセージコマンドが有効なら送信
// parameter
//		char	*pcData		[in]UTF-8データ(NULLターミネートなし)/NULL
//		int		iSize		[in]データサイズ
//		LPCSTR	szUserName	[in]UserName SJIS
// retun
//		TRUE/FALSE
//----------------------------------------------
BOOL CheckAndSendMessagePacket(char* pcData, int iSize, LPCSTR szUserName)
{
	BOOL	fRet = FALSE;

	if ((pcData != NULL) && (iSize > 0))
		fRet = SendMessagePacket(pcData, iSize, szUserName);
	return(fRet);
}

//----------------------------------------------
// function
//		メッセージパケットの送信
// parameter
//		char	*pcData		[in]UTF-8データ(NULLターミネートなし)
//		int		iSize		[in]データサイズ(チャットメッセージの文字数)
//		LPCSTR	szUserName	[in]UTF-8ユーザ名(NULLターミネートあり)
// retun
//		なし
//----------------------------------------------
BOOL	 SendMessagePacket(char* pcData, int iSize, LPCSTR szUserName)
{
	MsgDataRec* pMsgData = NULL;
	WORD		wDataLen = (WORD)sizeof(UserInfoRec) + iSize;			//	ヘッダを除いたデータ長
	int			iPacketSize = sizeof(HeaderRec) + wDataLen;		// パケットサイズ
	BOOL		fRet = FALSE;

	pMsgData = (MsgDataRec*)calloc(iPacketSize, sizeof(BYTE));	// パケット全体のエリアを確保

	memcpy(pMsgData->header.bMagicData, MAGIC_STRING, strlen(MAGIC_STRING));
	pMsgData->header.wCommand = htons(CMD_MSG_DATA);			// htons ホストオーダーからネットワークオーダーへの変換(2Byte)
	pMsgData->header.wDataLen = htons(wDataLen);

	DISABLE_C4996
	strcpy(pMsgData->UserInfo.szUserName, szUserName);			// ユーザ名
	ENABLE_C4996
	memcpy(pMsgData->bMsgData, pcData, iSize);					// メッセージ
	fRet = m_pCSendRecvThread->SetSendData((char*)pMsgData, iPacketSize);

	SAFE_FREE(pMsgData)
	return(fRet);
}

//-----------------------------------------------------------------
// Function
//		指定されたインターフェースの情報を取得する
// Parameter
//		int		iFamily		[in]アドレスファミリ
//		LPCSTR	szIFName	[in]インターフェース名
//		DWORD	*pdwIndex	[in/out]インターフェースインデックス
// Return
//		指定条件のインターフェースの最初のユニキャストアドレス
//-----------------------------------------------------------------
sockaddr *GetInterfaceInfo(int iFamily, LPCSTR szIFName, DWORD *pdwIndex)
{
	int		iRet = 0;
	struct ifaddrs *pifaddrs = NULL, *pifa;
	char	szAddr[NI_MAXHOST], szMask[NI_MAXHOST];
	sockaddr	*pAddrRet = NULL;

	*pdwIndex = 0;
	if (szIFName == NULL)
		goto L_END;
// アダプタリストの取得
	if (getifaddrs(&pifaddrs) == -1)
		goto L_END;
// アダプタ一覧を出力
	for (pifa = pifaddrs; pifa != NULL; pifa = pifa->ifa_next)
	{
		if (pifa->ifa_addr == NULL)
			continue;
		if ((pifa->ifa_addr->sa_family != AF_INET) && (pifa->ifa_addr->sa_family != AF_INET6))
			continue;

		fprintf(stderr, "Name:%s\n", pifa->ifa_name);
// インターフェース名が指定のものでなければ、次へ
		if (strcmp(pifa->ifa_name, szIFName) != 0)
			continue;
// ファミリが指定のものでなければ次へ
		if (pifa->ifa_addr->sa_family != iFamily)
			continue;

// 名前からインデックスを取得する
		*pdwIndex = if_nametoindex(pifa->ifa_name);
		fprintf(stderr, "Index:%d\n", *pdwIndex);
		if (pifa->ifa_addr->sa_family == AF_INET)
		{
			inet_ntop(AF_INET, &((struct sockaddr_in *)pifa->ifa_addr)->sin_addr, szAddr, sizeof(szAddr));
			inet_ntop(AF_INET, &((struct sockaddr_in *)pifa->ifa_netmask)->sin_addr, szMask, sizeof(szMask));
			fprintf(stderr, "  IPv4: %s netmask %s\n", szAddr, szMask);
			pAddrRet = (sockaddr *)calloc(1, sizeof(sockaddr_in));
			memcpy(pAddrRet, pifa->ifa_addr, sizeof(sockaddr_in));
		}
		else if (pifa->ifa_addr->sa_family == AF_INET6)
		{
			inet_ntop(AF_INET6, &((struct sockaddr_in6 *)pifa->ifa_addr)->sin6_addr, szAddr, sizeof(szAddr));
			inet_ntop(AF_INET6, &((struct sockaddr_in6 *)pifa->ifa_netmask)->sin6_addr, szMask, sizeof(szMask));
			fprintf(stderr, "  IPv6: %s netmask %s\n", szAddr, szMask);
			pAddrRet = (sockaddr *)calloc(1, sizeof(sockaddr_in6));
			memcpy(pAddrRet, pifa->ifa_addr, sizeof(sockaddr_in6));
		}
// 見つかったのでリターンする
		if (pAddrRet != NULL)
			goto L_END;
		fprintf(stderr, "\n");
	}

L_END:
	if (pifaddrs != NULL)
		freeifaddrs(pifaddrs);
	return(pAddrRet);
}
