#pragma once

#include	"stdThread.h"

#define		MAGIC_DATA_LEN		36
#define		MAGIC_STRING		"E2E6F7BF-42B1-4382-AF0B-1F452ED13EB6"

// チャットのコマンド
#define		CMD_MSG_DATA			100

// Client->Server(REQがあるのはRESを期待するもの)
#define		CMD_SEND_FILE_START_REQ	200		// 送信開始要求
#define		CMD_SEND_FILE_ING		201		// ファイル送信中
#define		CMD_SEND_FILE_END_REQ	202		// 送信完了要求
#define		CMD_SEND_FILE_ABORT_REQ	203		// 中断要求
#define		CMD_TEMPREHUM_DATA1		500		// ★温度・気圧・湿度 100倍して符号付き整数で送受信
#define		CMD_TEMPREHUM_DATA2		600		// ★温度・気圧・湿度 を文字列データとして送受信
#define		CMD_TEMPREHUM_DATA3		700		// ★温度・気圧・湿度 をJSONデータにして送受信

// Server->Client
#define		CMD_SEND_FILE_START_RES	1200	// 送信開始要求に対する応答
#define		CMD_SEND_FILE_END_RES	1202	// 送信完了要求に対する応答
#define		CMD_SEND_FILE_ABORT_RES	1203	// 中断要求に対する応答

#define		TIMEOUT_FOR_RES			(10000)	// ★応答までのタイムアウト時間msec

// ★Error Code
#define		ERR_SUCCESS				1		// 成功
#define		ERR_SEQUENCE			-1		// 状態遷移エラー
#define		ERR_FILE_OPEN			-2		// ファイルオープンエラー
#define		ERR_FILE_SEEK			-3		// ファイルシークエラー
#define		ERR_FILE_WRITE			-4		// ファイル書き込みエラー


#pragma pack(push ,1)				// 1バイト境界でデータを配置
// 共通ヘッダ部
typedef struct
{
	BYTE	bMagicData[MAGIC_DATA_LEN];		// 簡易不正アクセス防止機能用(WindowsのGUIDを使います)
	WORD	wCommand;						// ネットワークオーダー データの種別
	WORD	wDataLen;						// ネットワークオーダー(ヘッダを含まないデータ長)
} HeaderRec;

//==================================================
// メッセージ送信用パケット 共通ヘッダ部 + データ部
typedef struct
{
	char	szUserName[81];								// ★UserName UTF-8(nullターミネートあり)
} UserInfoRec;

typedef struct
{
	HeaderRec	header;
	UserInfoRec	UserInfo;
	BYTE		bMsgData[1];							// nullターミネイトを含まないデータ
} MsgDataRec;
//==================================================

//==================================================
// ファイル送信用パケット群
//==================================================
#define		FILE_NAME_LEN		256				// フォルダを含まない(今回対象とするのはカレントディレクトリにあるファイル)
#define		FILE_SIZE_DIGIT		10				// ファイル長は文字列にします(今回はlongの範囲 2GByte)
//------------------------------
// データ部 ファイル送信開始要求
//------------------------------
typedef struct
{
	char	szFileSize[FILE_SIZE_DIGIT + 1];	// ファイル長文字列(nullターミネートを含む)
	char	szFileName[FILE_NAME_LEN + 1];		// ファイル名文字列(nullターミネートを含む)UTF-8
} FileInfoRec;

typedef struct
{
	HeaderRec	header;
	FileInfoRec	FileInfo;
} SendFileStartReqRec;

//------------------------------
// データ部 ファイル送信(ファイルの途中、最後)
//------------------------------
typedef struct
{
	char	szOffset[FILE_SIZE_DIGIT + 1];
	char	szSize[FILE_SIZE_DIGIT + 1];
} SendFileStatRec;

typedef struct
{
	HeaderRec		header;
	SendFileStatRec	Stat;
	BYTE		bData[1];
} SendFileIngRec, SendFileEndReqRec;

//------------------------------
// データ部 ファイル送信中断要求
//------------------------------
typedef struct
{
	HeaderRec	header;
} SendFileAbortReqRec;

//------------------------------
// データ部 ファイル送信要求に対する応答
//------------------------------
typedef struct
{
	HeaderRec	header;
	BYTE		bStatus;						// ★1:成功 -XX:失敗
} SendFileResRec;

//==================================================
// ★温度・気圧・湿度の送信案1パケット 共通ヘッダ部 + データ部
typedef struct
{
	BYTE		bTem[4];			// 温度*100
	BYTE		bPre[4];			// 気圧*100
	BYTE		bHum[4];			// 湿度*100
} TemPreHumData1Rec;

typedef struct
{
	HeaderRec				header;
	UserInfoRec				UserInfo;
	TemPreHumData1Rec		TemPreHum;
} TemPreHum1Rec;

//==================================================
// ★温度・気圧・湿度の送信案2パケット 共通ヘッダ部 + データ部
typedef struct
{
	char		szTem[11];			// 温度
	char		szPre[11];			// 気圧
	char		szHum[11];			// 湿度
} TemPreHumData2Rec;

typedef struct
{
	HeaderRec				header;
	UserInfoRec				UserInfo;
	TemPreHumData2Rec		TemPreHum2;
} TemPreHum2Rec;

//==================================================
// ★温度・気圧・湿度の送信案3パケット 共通ヘッダ部 + データ部
typedef struct
{
	HeaderRec				header;
	UserInfoRec				UserInfo;
	BYTE					bJson[1];						// nullターミネイトを含まないデータ(メッセージと同様文字列が入る)
} TemPreHum3Rec;

#pragma pack(pop)				// 境界をもとに戻す

