#include	"stdThread.h"

BOOL	gfEnableESC = FALSE;

//-----------------------------------------------------------------
// Function
//		dwNow と dwPast の間の経過を求める
// Parameter
//		なし
// Return
//		経過
//-----------------------------------------------------------------
DWORD GetdwInterval(DWORD dwNow, DWORD dwPast)
{
	DWORD	dwInterval;

	if(dwNow >= dwPast)
		dwInterval = dwNow - dwPast;
	else
		dwInterval = dwNow + ((DWORD)(-1) - dwPast);
	return(dwInterval);
}

//-----------------------------------------------------------------
// Function
//		システムタイム(msec)を取得する
// Parameter
//		なし
// Return
//		なし
//-----------------------------------------------------------------
DWORD timeGetTime()
{
	DWORD			dwTime = 0;
	struct timeval 	tv;

	if(gettimeofday(&tv, NULL) == -1) 
		goto L_END;
	dwTime = tv.tv_sec * 1000 + tv.tv_usec / 1000;
L_END:
	return(dwTime);
}


//----------------------------------------------
// function
//		Windowsのkbhitと同様の機能をターミナルの
//		モードを変更することで実現したもの
// parameter
//		なし
// return
//    0:入力なし 1:入力あり
//----------------------------------------------
int kbhit(void)
{
	struct termios oldt, newt;
	int		ch = 0;;
	int		oldf;
	int		buf[81] = { 0 };
	pollfd	fds[1];

	// 現在のttyのモードを取得
	tcgetattr(STDIN_FILENO, &oldt);
	newt = oldt;

	// non-canonical mode、エコーモードなしに
	newt.c_lflag &= ~(ICANON | ECHO);
	// ttyモード即時変更(ちょっと時間が掛かる?)
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);

	fds[0].fd = STDIN_FILENO;					// 標準入力が調査対象
	fds[0].events = POLLIN;

	poll(fds, 1, 100);
	if (fds[0].revents & POLLIN)				// キー入力があった
	{
		read(STDIN_FILENO, buf, sizeof(buf));
		ch = buf[0];
	}
	// ttyのモードを元に戻す
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if (ch != 0)
	{
		// 入力された文字をストリームに戻す
		ungetc(ch, stdin);
		return 1;
	}
	return 0;
}

//----------------------------------------------
// function
//		Windowsのgetchと同様の機能をターミナルの
//		モードを変更することで実現したもの
// parameter
//		なし
// return
//    EOF/入力文字(エコーなし)
//----------------------------------------------
int getch(void)
{
	struct termios oldattr, newattr;
	int ch;

	// 現在のttyのモードを取得
	tcgetattr(STDIN_FILENO, &oldattr);
	newattr = oldattr;
	// 大文字変換、エコーモードなしに
	newattr.c_lflag &= ~(ICANON | ECHO);
	// ttyモード即時変更
	tcsetattr(STDIN_FILENO, TCSANOW, &newattr);
	// キー入力取得
	ch = getchar();
	// ttyのモードを元に戻す
	tcsetattr(STDIN_FILENO, TCSANOW, &oldattr);
	return ch;
}


#ifdef _NETWORK_APP
//----------------------------------------------
// function
//		エラーコードとエラーメッセージを表示します
// parameter
//		LPCSTR pszTitle	[in]メッセージにつけるタイトル
// return
//		エラーコード
//----------------------------------------------
int DispNetErrorMsg(LPCSTR pszTitle)
{
	fprintf(stderr, "%s %d %s\n", pszTitle, errno, strerror(errno));
	return(errno);
}
#endif

//-----------------------------------------------------------------
// Function
//		Sleep関数 Windowsと互換のため
// Parameter
//		DWORD dwMilliseconds
// Return
//		なし
//-----------------------------------------------------------------
void Sleep(DWORD dwMilliseconds)
{
	usleep(dwMilliseconds * 1000);
}

//----------------------------------------------------------------------
// function
//		ESCシーケンスによる画面制御を有効にする
//		Windowsと互換用(フラグを立てるだけ)
// parameter
//		FILE		*stream		[in]有効にしたいファイルストリーム
// return
//		TRUE/FALSE
//----------------------------------------------------------------------
BOOL EnableESC(FILE* stream)
{
	gfEnableESC = TRUE;
	return(TRUE);
}

//----------------------------------------------------------------------
// function
//		ESCシーケンスによるカーソル移動
// parameter
//		int		iCol		[in]列
//		int		iRow		[in]行
//		int		iErase		[in]0:消さない 1:行末まで消す 2:最後まで消す
// return
//		TRUE/FALSE
//----------------------------------------------------------------------
void Locate(int iCol, int iRow, int iErase)
{
	if (gfEnableESC == FALSE)
		return;
	switch (iErase)
	{
	case 1:
		fprintf(stdout, "\x1b[%d;%dH\x1b[0K", iRow, iCol);
		break;
	case 2:
		fprintf(stdout, "\x1b[%d;%dH\x1b[0J", iRow, iCol);
	default:
		fprintf(stdout, "\x1b[%d;%dH", iRow, iCol);
		break;
	}
}

//-----------------------------------------------------------------
// Function
//		四捨五入する
// Parameter
//		double		dbBasic			四捨五入する数字
//		int			point			四捨五入後の小数点以下の桁数
// Return
//		四捨五入した数字
//-----------------------------------------------------------------
double	RoundOff(double dbBasic, int point)
{
	double	dbAdd, dbreturn, dbData;
	double	dbbase = 1;
	DWORD	dwi;
	double	dbf, dbtemp;

	dbData = (dbBasic < 0) ? -dbBasic : dbBasic;

	dbbase /= pow(10, point + 1);
	dbAdd = dbData + 5 * dbbase;

	dwi = (DWORD)dbAdd;
	dbf = dbAdd - dwi;

	DWORD dwTemp = (DWORD)(dbf * pow(10, point));
	dbtemp = dwTemp / pow(10, point);

	dbreturn = (dbBasic < 0) ? -(dwi + dbtemp) : (dwi + dbtemp);

	return dbreturn;
}
